{-
Copyright (c) 2017 Ryan Olson

GNU GENERAL PUBLIC LICENSE
    Version 3, 29 June 2007

This file is part of elm-time-range.

elm-time-range is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

elm-time-range is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with elm-time-range.  If not, see <http://www.gnu.org/licenses/>.
-}

module TimeRange
    exposing
        ( TimeRange
        , fromStartAndEnd
        , start
        , end
        , isContainedWithin
        , overlaps
        , difference
        )

{-| A library for representing time ranges in an elegant manner.

# Definition
@docs TimeRange, fromStartAndEnd

# Accessors
@docs start, end

# Common Helpers
@docs difference, isContainedWithin, overlaps
-}

import Time exposing (Time)


{-| Representation of a time range.
-}
type TimeRange
    = TimeRange
        { start : Time
        , end : Time
        }


{-| Construct a time range from start and end times.
-}
fromStartAndEnd : Time -> Time -> TimeRange
fromStartAndEnd start end =
    TimeRange
        { start = start
        , end = end
        }


{-| Get the start time for the range.
-}
start : TimeRange -> Time
start (TimeRange { start, end }) =
    start


{-| Get the end time for the range.
-}
end : TimeRange -> Time
end (TimeRange { start, end }) =
    end


{-| Allows you to find out if a given `Time` is contained within a `TimeRange`.
-}
isContainedWithin : Time -> TimeRange -> Bool
isContainedWithin time timeRange =
    start timeRange < time && time <= end timeRange


{-| Allows you to find out if two `TimeRange`s overlap.
-}
overlaps : TimeRange -> TimeRange -> Bool
overlaps timeRange1 timeRange2 =
    isContainedWithin (start timeRange2) timeRange1
        || isContainedWithin (end timeRange1) timeRange2


{-| Get the `Time` difference between the start and end time in the time range.
-}
difference : TimeRange -> Time
difference timeRange =
    abs <| end timeRange - start timeRange
